/* TNC-Wchter
 * berwacht die DATA-Leitung eines TNC2. Leuchtet die
 * DATA-LED fr mehr als X Sekunden wird ein kurzer Warnton
 * abgegeben. Leuchtet die Data-LED fr mehr als 10 Minuten
 * werden zwei kurze Warntne abgegeben. Ist die Data-LED aus
 * schlft der ATtiny13.
 *
 *
 * Takt              4,8 MHz
 * Vorteiler (Timer):256
 * 4.8 MHz / 256 = 18750 Hz 
 */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#define SPEAKER PB0
#define LED PB1
#define INP PB2

// Einheit 1/10 Sekunde; mu kleiner 65536 sein!
#define WARTE_MAXIMUM 3500 
#define WARTE_KURZ 20
#define WARTE_LANG 3000 // 5 Minuten
#define WARTE_TOL 10
#define BEEPLENGTH 200
#define PAUSELENGTH 300


#define TIMSK0_VAL (1 << TOIE0) | (0 << OCIE0A) | (0<<OCIE0B)
#define TCCR0A_PWM   (0 << COM0A1) | \
                     (1 << COM0A0) | \
                     (0 << COM0B1) | \
                     (0 << COM0B0) | \
                     (0 << WGM01) | \
                     (1 << WGM00)

  // Vorteiler 256
#define TCCR0B_PWM   (0 << FOC0A) | \
                     (0 << FOC0B) | \
                     (1 << WGM02) | \
                     (1 << CS02) | \
                     (0 << CS01) | \
                     (0 << CS00)


volatile uint16_t counter;
void beep(uint8_t anzahl);

/* Der Pin-Change-Interrupt ist der einzige Wecker,
 * der den ATtiny aufwecken kann, wenn es im Sleep-
 * Modus ist und der Timer-Interrupt abgestellt ist.
 *
 * Wenn der Input-Pin High ist wird der Timer gestartet,
 * sobald der Pin wieder Low ist wird er gestoppt.
 */

SIGNAL(SIG_PIN_CHANGE0)
{
  if(PINB & (1 << INP))
  {
    TIMSK0  =  TIMSK0_VAL;// Timer an
  }
  else
  {
    counter = 0;
    TIMSK0=0;            // Timer aus

  }
}



// Timer-Interrupt 
SIGNAL(SIG_OVERFLOW0)      //overflow interrupt vector
{
  counter++;
  //PORTB ^= (1 << LED);
}




/* Die Hauptschleife prft, ob es einen Grund zum Piepen gibt und legt
 * den ATtiny danach sofort wieder schlafen.
 */
int main()
{
  // Ausgnge
  DDRB |= (1 << LED) | (1 << SPEAKER);
    
  //Pin Change Interrupt an (S. 46 Datenblatt):
  GIMSK |= (1 << PCIE);
  //Pin Chance Interrupt nur an Pin PB2:
  PCMSK = (1 << PCINT2);


  // Timer initialisieren (S. 74 Datenblatt):
  // Timer-Register OCR0B
   //TIMSK0  =  (1 << TOIE0) | (0 << OCIE0A) | (0<<OCIE0B);  //set 8-bit Timer/Counter0 Overflow Interrupt Enable        
   //TCCR0A = 0x00;

  // Prescaler 256
   TCCR0B |= (1 << CS02) | (0<<CS01) | (1<<CS00);
  
   // Startwert
   TCNT0 = 0x00;

  
   set_sleep_mode(SLEEP_MODE_IDLE);
 
   sei();

   counter=0;
   while(1) 
   {
     if((counter>=WARTE_KURZ) && (counter< (WARTE_KURZ+WARTE_TOL)))
     {
       beep(2);
       counter+= WARTE_TOL;
     }

     if((counter>=WARTE_LANG) && (counter< (WARTE_LANG+WARTE_TOL)))
     {
       beep(5);
       counter+= WARTE_TOL;
     }
 
     if(counter>=WARTE_MAXIMUM)
     {
       TIMSK0 = 0;            // Timer aus
     }   


     sleep_mode();
   } // Ende Hauptschleife

 return 0;
}

/* Gibt eine Anzahl von Pieptnen per PWM aus
 * Da fr die PWM der einzige Timer verwendet wird,
 * wird der globale Zeitzhler angehalten und an der
 * gleichen Stelle spter fortgesetzt.
 */
void beep(uint8_t anzahl)
{ 
  char temp1 = TCCR0A;
  char temp2 = TCCR0B;
  char temp3 = OCR0A;
  char temp4 = counter;

  // Initialisierung PWM
  OCR0A = 0x03;
  TCCR0A = TCCR0A_PWM;

  while(anzahl)
  {
    TCCR0B =  TCCR0B_PWM;
    _delay_ms(BEEPLENGTH);
    TCCR0B = 0;
    _delay_ms(PAUSELENGTH);
    anzahl--;
  }
 
  TCCR0B = temp2;
  TCCR0A = temp1;
  OCR0A  = temp3;
  counter=temp4;
 }

